# -*- perl -*-

package Bundle::DBD::mysql;

$VERSION = 4.002;

1;

__END__

=head1 NAME

Bundle::DBD::mysql - A bundle to install Perl drivers for MySQL

=head1 SYNOPSIS

C<perl -MCPAN -e 'install Bundle::DBD::mysql'>

=head1 CONTENTS

DBI

DBD::mysql

=head1 DESCRIPTION

This bundle includes all that is needed to connect with a MySQL server using
DBI.

=cut
